package com.ippopay.ippo_pay;

import android.content.Context;
import android.util.Log;

import android.app.Activity;

import androidx.annotation.NonNull;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.ippopay.core.IppoPayListener;
import com.ippopay.core.IppoPayLog;
import com.ippopay.core.IppoPayPay;
import com.ippopay.models.OrderData;

import org.json.JSONObject;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * IppoPayPlugin
 */
public class IppoPayPlugin implements FlutterPlugin, MethodCallHandler
        , ActivityAware, IppoPayListener {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private MethodChannel channel;

    private Context context;

    private Result result;

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "ippo_pay");
        channel.setMethodCallHandler(this);
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, final @NonNull Result result) {
        try {
            if (call.method.equals("initSDK")) {
                IppoPayPay.init(context, call.argument("public_key").toString());
                // result.success("Ippopay Init Success");
            } else if (call.method.equals("makePayment")) {
                this.result = result;
                IppoPayLog.setLogVisible(true);
                String orderObj = call.argument("order_data").toString();
                Log.d("Method Calls:::", "orderObj:::" + orderObj);
                IppoPayPay.setPaymentListener(this);
                IppoPayPay.makePayment(getGSONObj().fromJson(orderObj, OrderData.class));
            } else if (call.method.equals("setLogVisible")) {
                IppoPayLog.setLogVisible(Boolean.parseBoolean(call.argument("visible").toString()));
            } else {
                result.notImplemented();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }

    @Override
    public void onDetachedFromActivity() {
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        this.context = binding.getActivity();
    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {
    }

    Gson getGSONObj() {
        return new GsonBuilder().create();
    }

    @Override
    public void onTransactionSuccess(String transactionId) {
        sendCallBack(true, transactionId, "Payment Success");
    }

    @Override
    public void onTransactionFailure(final String error, String transaction_id) {
        sendCallBack(false, transaction_id, error);
    }

    @Override
    public void onTransactionCancelled() {
        sendCallBack(false, null, "Payment Cancelled");
    }

    private void sendCallBack(boolean success, String transactionId, String message) {
        try {
            if (result != null) {
                JSONObject jsonObj = new JSONObject();
                jsonObj.put("success", success);
                jsonObj.put("message", message);
                jsonObj.put("transaction_id", transactionId);
                result.success(jsonObj.toString());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
