import Flutter
import UIKit
import IppoPay

public class SwiftIppoPayPlugin: NSObject, FlutterPlugin, IppopayDelegate {

    var callBack: FlutterResult?
      
    public static func register(with registrar: FlutterPluginRegistrar) {
      let methodChannel = FlutterMethodChannel(name: "ippo_pay", binaryMessenger: registrar.messenger())
      let instance = SwiftIppoPayPlugin()
      registrar.addMethodCallDelegate(instance, channel: methodChannel)
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
      if call.method == "initSDK" {
          if let args = call.arguments as? Dictionary<String, Any>,
            let key = args["public_key"] as? String {
            Ippopay.initSDK(publicKey: key, withDelegate: self)
          }
      } else if call.method == "makePayment" {
          callBack = result
          if let args = call.arguments as? Dictionary<String, Any>,
            let orderString = args["order_data"] as? String {
              let dictValue = convertToDictionary(text: orderString)
              let order = OrderData()
              order.orderId = dictValue?["order_id"] as? String
              order.currencyCode = dictValue?["currency"] as? String
              order.orderAmount = dictValue?["amount"] as? Double
              let customerValue = dictValue?["customer"] as? [String: Any]
              let customer = Customer()
              customer.customerEmail =  customerValue?["email"] as? String
              customer.customerName =  customerValue?["name"] as? String
              customer.customerPhoneNumber =  (customerValue?["phone"] as? [String: Any])?["national_number"] as? String
              order.customer = customer
            Ippopay.makePayment(orderData: order)
          }
      } else if call.method == "setLogVisible" {
        if let args = call.arguments as? Dictionary<String, Any>,
            let debug = args["debug"] as? Bool {
              PayLog.setLogVisible(debug: debug)
          }
      }
    }
      
      func convertToDictionary(text: String) -> [String: Any]? {
          if let data = text.data(using: .utf8) {
              do {
                  return try JSONSerialization.jsonObject(with: data, options: []) as? [String: Any]
              } catch {
                  print(error.localizedDescription)
              }
          }
          return nil
      }

      public func onPaymentError(descriptionOfError: String) {
          let result = ["success" : false, "message" : descriptionOfError] as [String : Any]
          if let theJSONData = try? JSONSerialization.data(withJSONObject: result,options: []) {
              let theJSONText = String(data: theJSONData, encoding: .utf8)
              callBack!(theJSONText!)
          }
      }

      public func onPaymentSuccess(paymentId: String) {
          let result = ["success" : true, "message" : "Payment Successful", "transaction_id": paymentId] as [String : Any]
          if let theJSONData = try? JSONSerialization.data(withJSONObject: result,options: []) {
              let theJSONText = String(data: theJSONData, encoding: .utf8)
              callBack!(theJSONText!)
          }
      }
}
